body { background-color:#666; color:#FFF; font:11px/1.5 Arial, sans-serif; margin:0; padding:30px 0 0 30px; }
h1 { font-size:18px; margin:0 0 20px; }
a { color:#000; }

.clear { clear:both; font-size:1px; line-height:1px; }

#overlay { background:#666; height:100%; left:0; position:absolute; top:0; width:100%; z-index:2000; }
#overlay #preloader { background:url(loader_bg.gif) no-repeat; height:50px; left:50%; line-height:50px; margin:-25px 0 0 -25px; position:absolute; text-align:center; top:50%; width:50px; }
#overlay #preloader img { margin:0; vertical-align:middle; }

.ui-sortable { background-color:#FFF; border:1px solid #555; color:#222; margin:0 15px 15px 0; padding:0 10px 10px; width:175px; }
.ui-sortable h2 { background-color:#555; border-top:3px solid #666; color:#FFF; font-size:11px; margin:0 -10px 10px; line-height:2; padding:0 10px; position:relative; }

dl.sort { color:#222; margin:10px 0; }
#uidemo dl.first { margin-top:0; }
#uidemo dl.last { margin-bottom:0; }

dl.sort dt { background-color:#666; color:#FFF; cursor:move; height:2em; line-height:2; padding:0 6px; position:relative; }
dl.sort dd { background-color:#FFF; margin:0; padding:3px 6px; }

.ui-sortable-helper { width:175px; }
.placeholder { border:1px dashed #AAA; }

span.options { cursor:default; font-size:1px; line-height:1px; position:absolute; }
span.options a { background-color:#FFF; cursor:pointer; display:block; float:left; text-indent:-9000px; }
span.options a.add { background: url(add.png) no-repeat; }

.ui-sortable h2 span.options { right:-12px; top:5px; width:30px; }
.ui-sortable h2 span.options a { height:12px; width:30px; }

dl.sort dt span.options { right:5px; top:5px; width:27px; }
dl.sort dt span.options a { height:12px; width:12px; }
dl.sort dt span.options a.down { background: url(down.png) no-repeat; }
dl.sort dt span.options a.up { margin-right:3px; background: url(up.png) no-repeat; }
dl.sort dt span.options a.disabled { background:#555 !important; cursor:default; }

#container { float:left; }
#header { width:638px; }
#content { float:left; width:400px; }
#sidebar { float:left; width:200px; }
#footer { width:638px; }
#meta { float:left; }

#trashcan p { margin:0; }
